/*  shogiopening
 *  Copyright (C) 2000 Sverre Hvammen Johansen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include "interface.h"
#include "graph.h"
#include "fileio.h"
#include "move.h"
#include "draw.h"
#include "support.h"
#include "scanner.h"

GtkWidget *fileselection;

#define OPEN 1
#define SAVE 2
#define MERGE 3
#define SAVE_GAME 4

int fileseltype=0;
char *default_filename=NULL;
GnomeAppBar *appbar1;

void update_interface(t_graph *g)
{
  if (g) g_slist_pop_allocator();
  gnome_appbar_set_progress (appbar1, progress());
  while (gtk_events_pending())
    gtk_main_iteration();
  if (g) g_slist_push_allocator(g->gslist);
}

void init_file()
{
  appbar1= GNOME_APPBAR (lookup_widget(app1, "appbar1"));
  fileselection = create_fileselection ();
  gtk_file_selection_complete (GTK_FILE_SELECTION (fileselection), 
			       PKGDATADIR "/*.sdb");
}

t_graph *new_file(t_graph *g)
{
  default_filename=NULL;
  if (g) free_graph (g);
  g= new_main_graph ();
  g->current_node= g->main;
  draw_board(g, 0, 0);
  return (g);
}

void open_file()
{
  fileseltype=OPEN;
  gtk_widget_show (fileselection);
}

void save_file(t_graph *g)
{
  if (default_filename)
    save_graph(default_filename, g->main);
  else
    save_as_file();
}

void save_as_file()
{
  fileseltype=SAVE;
  gtk_widget_show (fileselection);
}

void save_game()
{
  fileseltype=SAVE_GAME;
  gtk_widget_show (fileselection);
}

void merge_file()
{
  fileseltype=MERGE;
  gtk_widget_show (fileselection);
}

t_graph * open_file_name (t_graph *g, char *name, char open)
{
  g= open_graph(g, name, open);
  g->current_node= g->main;
  gnome_appbar_set_progress (appbar1, 0.0);
  draw_board(g, 0, 0);  
  return g;
}

t_graph *ok_file(t_graph *g)
{
  gtk_widget_hide (fileselection);
  switch (fileseltype)
    {
    case OPEN:
    case MERGE:
      gnome_appbar_push (GNOME_APPBAR(lookup_widget(app1, "appbar1")), 
			 fileseltype==OPEN?"Opening file"
			 :"Merging files");
      draw_board (NULL, 0, 0);
      while (gtk_events_pending())
	gtk_main_iteration();

      g= open_file_name (g, 
			 default_filename=gtk_file_selection_get_filename
			 (GTK_FILE_SELECTION(fileselection)),
			 fileseltype==OPEN);

      if (fileseltype==MERGE) g->unsaved= 1;

      gnome_appbar_pop (GNOME_APPBAR(lookup_widget(app1, "appbar1")));
      break;
    case SAVE:
      save_graph(gtk_file_selection_get_filename
			(GTK_FILE_SELECTION(fileselection)), g->main);
      g->unsaved= 0;
      break;
    case SAVE_GAME:
      save_path(gtk_file_selection_get_filename
			(GTK_FILE_SELECTION(fileselection)), g->main);
      break;
    }
  return g;
}

void cancel_file()
{
  gtk_widget_hide (fileselection);
}
