/*  shogiopening
 *  Copyright (C) 2000 Sverre Hvammen Johansen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#define BLACK 16
#define WHITE 0
#define PROMOTED 8
#define PIECEMASK 7
#define PAWN 1
#define ROOK 2
#define BISHOP 3
#define LANCE 4
#define KNIGHT 5
#define SILVER 6
#define GOLD 7
#define KING 8

#define WHITE_HAND 0
#define BLACK_HAND 10

typedef struct
{
  char *sente;
  char *gote;
  char *date;
  char *event;
  char *opening;
  char *result;
}
t_data;

typedef struct s_node t_node;

typedef struct
{
  int update_seq;
  int pathl;
  int nodes;
  int leaves;
} t_compute;

struct s_node {
  int node_number;
  short mark;
  char leaf;
  t_data *data;
  char *note;
#ifdef COMPRESS_BOARD
  int board[9];
#else
  char board[10][11];
#endif
  GSList* froms;
  GSList* tos;
  t_node *prev;
  t_node *next;
  t_compute *compute;
};
#define TURN(b) b[9][0]

typedef struct {
  int x;
  int y;
} t_cord;

typedef struct {
  char *move;
  t_node *to_node;
  t_node *from_node;
  t_cord from;
  t_cord to;
  t_cord extra;
  int line;
  char bad;
} t_arc;

typedef struct {
  t_node *main, *current_node;
  int update_seq;
  GMemChunk *node, *arc, *data, *compute;
  GAllocator *gslist;
  t_data *current_data;
  char unsaved;
} t_graph;

t_data *new_data (t_graph *g);
extern t_arc *new_arc(t_graph *g, char *move, t_node *node);
extern t_graph *open_graph(t_graph *main, char *name, char merge);
extern void save_graph(char *name, t_node *n);
extern void save_path(char *name, t_node *n);
extern void graph_init(void);
extern t_node *add_one_node (t_graph *g, t_node *n,
			     int x1, int y1, 
			     int x2, int y2, char auto_promote, 
			     char *move, char board[10][11]);
extern t_graph *merge_graphs (t_graph *g1, t_graph *g2);
extern t_node *new_node(t_graph *g, long number, char *note);
extern t_graph *new_graph (void);
extern t_graph *new_main_graph (void);
extern void free_nodes(t_graph *g, t_node *n);
extern void free_graph(t_graph *g);
extern int move_number (t_node *n);
extern void free_node (t_graph *g, t_node *n);
extern void mark_nodes(t_node *n, int m);
extern void prune_graph(t_graph *g, t_node *n, int max_dist);
extern void add_from_next_nodes (t_node *n);

/* In parser.y */
extern t_graph *build_graph(char *name, gboolean *foldp);

