%{
/*  shogiopening
 *  Copyright (C) 2000 Sverre Hvammen Johansen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <glib.h>
#include "graph.h"
#include "parser.h"
#include <string.h>
#include "hash.h"
int yyline;
%}
%%
["][^"]*["]|[{][^}]*[}] {  { int i, num=0; 
                             for (i=0;yytext[i];i++) 
                               if (yytext[i]=='\n')yyline++;
                               else if (yytext[i]=='"')
                                 {
                                   if (num++ %2) yytext[i]='\'';
                                   else
                                     yytext[i]='`';
                                 }
			     yytext[strlen(yytext)-1]='\0';
                           }
                           yylval.string= 
                             find_string_in_hash_table (&yytext[1]);
                           return(TEXT);}
[0-9]+\.?		{yylval.val=atoi(yytext);return(NUM);}
NODE			{return(NODE);}
:A-KOORDINAT		{return(AKOR);}
:B-KOORDINAT		{return(BKOR);}
:LEAF			{return(LEAF);}
:NOTE			{return(NOTE);}
:FROMS			{return(FROMS);}
:TOS			{return(TOS);}
:POSITION		{return(POSITION);}
:BOOK-DELAY		{return(BOOKDEL);}
:BAD			{return(BAD);}
:SENTE			{return(SENTE);}
:GOTE			{return(GOTE);}
:DATE			{return(DATE);}
:EVENT			{return(EVENT);}
:OPENING		{return(OPENING);}
:RESULT			{return(RESULT);}
nil			{return(NIL);}
NIL			{return(NIL);}
"("			{return('(');}
")"			{return(')');}
"#"			{return('#');}
"["			{return('[');}
"]"			{return(']');}
"="			{return('=');}
"."			{return('.');}
":"			{return(':');}
"\\"			{return('\\');}
"l"			{return('l');}
"n"			{return('n');}
"s"			{return('s');}
"g"			{return('g');}
"k"			{return('k');}
"r"			{return('r');}
"b"			{return('b');}
"p"			{return('p');}
"L"			{return('L');}
"N"			{return('N');}
"S"			{return('S');}
"G"			{return('G');}
"R"			{return('R');}
"B"			{return('B');}
"P"			{return('P');}
"W"			{return('W');}
"A"			{return('A');}
[\n]			{yyline++; }
"Sente"|"Black"		{return(SENTE);} 
"Gote"|"White"		{return(GOTE);} 
"Event"			{return(EVENT);} 
"Date"			{return(DATE);} 
"Round"			{return(ROUND);} 
"Site"			{return(SITE);} 
"Result"		{return(RESULT);} 
"Opening"		{return(OPENING);}
"Name"			{return(NAME);}
"Email"			{return(EMAIL);}
"Country"			{return(COUNTRY);}
"Black_grade"			{return(BLACK_GRADE);}
"White_grade"			{return(WHITE_GRADE);}
"Comment"			{return(COMMENT);}
"Source"			{return(SOURCE);}
"Venue"			{return(VENUE);}
"Proam"			{return(PROAM);}
\+?[LNSGRBPK][-123456789abcdefghi\+\!\=\'x\?\*]+ {
                           yylval.string= find_string_in_hash_table(yytext);
			   return(MOVE); }
[0-9]+\/[0-9]+		{ }
"0-1"|"1-0"		{ }
[ \t\r]+		{ }
.			{ return(yytext[0]);}
%%

FILE *f;

int length_of_file;

char init_scanner(char *filename)
{
  yyline=1;

  f=fopen(filename, "r");
  fseek(f, 0, SEEK_END);
  length_of_file= ftell(f);
  fseek(f, 0, SEEK_SET);
  if (f == NULL) return(0);
  yyrestart (f);
  return(1);
}

double progress ()
{
  if (length_of_file)
    return (ftell(f)*1.0/length_of_file);
  else
    return 1.0;
}


int yywrap()
{
  return (1);
}

void cleanup_scanner()
{
  length_of_file= 0;
  if (yyin) fclose (yyin);
}

